@@ -42,7 +42,7 @@ have_func 'rb_errinfo', ruby_header
                      :target_build_dir => build_dir)
 end
 
-setup_win32(module_name, base_dir)
+setup_windows(module_name, base_dir)
 
 unless required_pkg_config_package([package_id, 1, 12, 0],
                                    :debian => "libatk1.0-dev",
