@@ -14,6 +14,7 @@
 # License along with this library; if not, write to the Free Software
 # Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
 
+require "gdk_pixbuf2"
 require "clutter"
 require "gst"
 
@@ -43,6 +44,8 @@ module ClutterGst
         remove_method(:init)
         remove_method(:const_missing)
       end
+      Gst.init if Gst.respond_to?(:init)
+      Clutter.init if Clutter.respond_to?(:init)
       loader = Loader.new(self, argv)
       loader.load("ClutterGst")
     end
@@ -61,11 +64,10 @@ module ClutterGst
     def pre_load(repository, namespace)
       init = repository.find(namespace, "init")
       arguments = [
-        1 + @init_arguments.size,
         [$0] + @init_arguments,
       ]
       error, returned_arguments = init.invoke(:arguments => arguments)
-      @init_arguments.replace(returned_arguments)
+      @init_arguments.replace(returned_arguments[1..-1])
       if error.to_i <= 0
         raise InitError, "failed to initialize Clutter-GStreamer: #{error.name}"
       end
