@@ -62,13 +62,12 @@ module ClutterGtk
     def pre_load(repository, namespace)
       init = repository.find(namespace, "init")
       arguments = [
-        1 + @init_arguments.size,
         [$0] + @init_arguments,
       ]
       error, returned_arguments = init.invoke(:arguments => arguments)
-      @init_arguments.replace(returned_arguments)
+      @init_arguments.replace(returned_arguments[1..-1])
       if error.to_i <= 0
-        raise InitError, "failed to initialize Clutter: #{error.name}"
+        raise InitError, "failed to initialize Clutter-GTK: #{error.name}"
       end
     end
 
