@@ -1,4 +1,4 @@
-# Copyright (C) 2012-2013  Ruby-GNOME2 Project Team
+# Copyright (C) 2012-2014  Ruby-GNOME2 Project Team
 #
 # This library is free software; you can redistribute it and/or
 # modify it under the terms of the GNU Lesser General Public
@@ -16,6 +16,7 @@
 
 require "gobject-introspection"
 require "cairo-gobject"
+require "pango"
 
 base_dir = Pathname.new(__FILE__).dirname.dirname.expand_path
 vendor_dir = base_dir + "vendor" + "local"
@@ -26,7 +27,6 @@ GObjectIntrospection.prepend_typelib_path(vendor_girepository_dir)
 
 if vendor_dir.exist?
   require "atk"
-  require "pango"
   require "gdk_pixbuf2"
 end
 
@@ -54,11 +54,14 @@ module Clutter
       require "clutter/actor"
       require "clutter/actor-iter"
       require "clutter/animatable"
+      require "clutter/brightness-contrast-effect"
       require "clutter/cairo"
+      require "clutter/clutter"
       require "clutter/color"
       require "clutter/event"
       require "clutter/point"
       require "clutter/text"
+      require "clutter/text-buffer"
       require "clutter/threads"
     end
   end
@@ -79,11 +82,10 @@ module Clutter
     def pre_load(repository, namespace)
       init = repository.find(namespace, "init")
       arguments = [
-        1 + @init_arguments.size,
         [$0] + @init_arguments,
       ]
       error, returned_arguments = init.invoke(:arguments => arguments)
-      @init_arguments.replace(returned_arguments)
+      @init_arguments.replace(returned_arguments[1..-1])
       if error.to_i <= 0
         raise InitError, "failed to initialize Clutter: #{error.name}"
       end
@@ -91,6 +93,8 @@ module Clutter
       @base_module.const_set("Keys", @keys_module)
       @threads_module = Module.new
       @base_module.const_set("Threads", @threads_module)
+      @feature_module = Module.new
+      @base_module.const_set("Feature", @feature_module)
     end
 
     def post_load(repository, namespace)
@@ -140,6 +144,13 @@ module Clutter
         # ignore
       when /\Athreads_/
         define_module_function(@threads_module, $POSTMATCH, info)
+      when /\Afeature_/
+        method_name = rubyish_method_name(info, :prefix => "feature_")
+        case method_name
+        when "available"
+          method_name = "#{method_name}?"
+        end
+        define_module_function(@feature_module, method_name, info)
       else
         super
       end
