@@ -39,7 +39,7 @@ end
                      :target_build_dir => build_dir)
 end
 
-setup_win32(module_name, base_dir)
+setup_windows(module_name, base_dir)
 
 unless required_pkg_config_package(package_id,
                                    :debian => "libgdk-pixbuf2.0-dev",
