@@ -22,6 +22,10 @@
 #include "rbgprivate.h"
 
 static ID id_call;
+static ID id_puts;
+static ID id_unpack;
+
+static VALUE default_rs;
 
 #define RG_TARGET_NAMESPACE cIOChannel
 #define _SELF(s) ((GIOChannel*)RVAL2BOXED(s, G_TYPE_IO_CHANNEL))
@@ -415,8 +419,6 @@ rg_write(VALUE self, VALUE buf)
     return UINT2NUM(bytes_written);
 }
 
-static ID id_unpack;
-
 static VALUE
 rg_putc(VALUE self, VALUE thechar)
 {
@@ -666,7 +668,7 @@ ioc_puts_ary(VALUE ary, VALUE out, int recur)
         if (recur) {
             tmp = rb_str_new2("[...]");
         }
-        rb_io_puts(1, &tmp, out);
+        rb_funcall(out, id_puts, 1, tmp);
     }
     return Qnil;
 }
@@ -679,7 +681,7 @@ rg_puts(int argc, VALUE *argv, VALUE self)
 
     /* if no argument given, print newline. */
     if (argc == 0) {
-        rg_write(self, rb_default_rs);
+        rg_write(self, default_rs);
         return Qnil;
     }
     for (i=0; i<argc; i++) {
@@ -687,11 +689,7 @@ rg_puts(int argc, VALUE *argv, VALUE self)
             line = rb_str_new2("nil");
         }
         else {
-#ifdef HAVE_RB_CHECK_ARRAY_TYPE
-          line = rb_check_array_type(argv[i]);
-#else
-          line = rb_check_convert_type(argv[i], T_ARRAY, "Array", "to_ary");
-#endif
+          line = rbg_check_array_type(argv[i]);
           if (!NIL_P(line)) {
 #ifdef HAVE_RB_EXEC_RECURSIVE
             rb_exec_recursive(ioc_puts_ary, line, self);
@@ -705,7 +703,7 @@ rg_puts(int argc, VALUE *argv, VALUE self)
         rg_write(self, line);
         if (RSTRING_LEN(line) == 0 ||
             RSTRING_PTR(line)[RSTRING_LEN(line)-1] != '\n') {
-            rg_write(self, rb_default_rs);
+            rg_write(self, default_rs);
         }
     }
 
@@ -717,6 +715,7 @@ rg_print(int argc, VALUE *argv, VALUE out)
 {
     int i;
     VALUE line;
+    VALUE output_field_separator;
 
     /* if no argument given, print `$_' */
     if (argc == 0) {
@@ -724,9 +723,11 @@ rg_print(int argc, VALUE *argv, VALUE out)
         line = rb_lastline_get();
         argv = &line;
     }
+
+    output_field_separator = rb_gv_get("$,");
     for (i=0; i<argc; i++) {
-        if (!NIL_P(rb_output_fs) && i>0) {
-            rg_write(out, rb_output_fs);
+        if (!NIL_P(output_field_separator) && i>0) {
+            rg_write(out, output_field_separator);
         }
         switch (TYPE(argv[i])) {
           case T_NIL:
@@ -737,8 +738,8 @@ rg_print(int argc, VALUE *argv, VALUE out)
             break;
         }
     }
-    if (!NIL_P(rb_output_rs)) {
-        rg_write(out, rb_output_rs);
+    if (!NIL_P(output_field_separator)) {
+        rg_write(out, output_field_separator);
     }
 
     return Qnil;
@@ -752,8 +753,16 @@ Init_glib_io_channel(void)
     rb_include_module(RG_TARGET_NAMESPACE, rb_mEnumerable);
 
     id_call = rb_intern("call");
+    id_puts = rb_intern("puts");
     id_unpack = rb_intern("unpack");
 
+    default_rs = rb_str_new_cstr("\n");
+#ifdef HAVE_RB_GC_REGISTER_MARK_OBJECT
+    rb_gc_register_mark_object(default_rs);
+#else
+    rb_global_variable(&default_rs);
+#endif
+
     RG_DEF_METHOD(initialize, -1);
     RG_DEF_SMETHOD(open, -1);
     RG_DEF_METHOD(fileno, 0);
@@ -799,12 +808,9 @@ Init_glib_io_channel(void)
     rb_define_const(RG_TARGET_NAMESPACE, "STATUS_AGAIN", INT2NUM(G_IO_STATUS_AGAIN));
 
     /* GIOCondition */
-    rb_define_const(RG_TARGET_NAMESPACE, "IN", INT2NUM(G_IO_IN));
-    rb_define_const(RG_TARGET_NAMESPACE, "OUT", INT2NUM(G_IO_OUT));
-    rb_define_const(RG_TARGET_NAMESPACE, "PRI", INT2NUM(G_IO_PRI));
-    rb_define_const(RG_TARGET_NAMESPACE, "ERR", INT2NUM(G_IO_ERR));
-    rb_define_const(RG_TARGET_NAMESPACE, "HUP", INT2NUM(G_IO_HUP));
-    rb_define_const(RG_TARGET_NAMESPACE, "NVAL", INT2NUM(G_IO_NVAL));
+    /* Deprecated. Just for bacakward compatibility. Use
+     * GLib::IOCondition::* instead. */
+    G_DEF_CONSTANTS(RG_TARGET_NAMESPACE, G_TYPE_IO_CONDITION, "G_IO_");
 
     /* GIOFlags */
     rb_define_const(RG_TARGET_NAMESPACE, "FLAG_APPEND", INT2NUM(G_IO_FLAG_APPEND));
