@@ -1,20 +1,30 @@
 /* -*- c-file-style: "ruby"; indent-tabs-mode: nil -*- */
-/**********************************************************************
-
-  rbgobj_signal.c -
-
-  $Author: ssimons $
-  $Date: 2007/07/25 17:04:28 $
-  created at: Sat Jul 27 16:56:01 JST 2002
-
-  Copyright (C) 2002-2004  Ruby-GNOME2 Project Team
-  Copyright (C) 2002,2003  Masahiro Sakai
-
-**********************************************************************/
+/*
+ *  Copyright (C) 2011  Ruby-GNOME2 Project Team
+ *  Copyright (C) 2002-2004  Ruby-GNOME2 Project Team
+ *  Copyright (C) 2002,2003  Masahiro Sakai
+ *
+ *  This library is free software; you can redistribute it and/or
+ *  modify it under the terms of the GNU Lesser General Public
+ *  License as published by the Free Software Foundation; either
+ *  version 2.1 of the License, or (at your option) any later version.
+ *
+ *  This library is distributed in the hope that it will be useful,
+ *  but WITHOUT ANY WARRANTY; without even the implied warranty of
+ *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+ *  Lesser General Public License for more details.
+ *
+ *  You should have received a copy of the GNU Lesser General Public
+ *  License along with this library; if not, write to the Free Software
+ *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
+ *  MA  02110-1301  USA
+ */
 
 #include "rbgprivate.h"
 
-static VALUE cSignal;
+#define RG_TARGET_NAMESPACE cSignal
+
+static VALUE RG_TARGET_NAMESPACE;
 VALUE rbgobj_signal_wrap(guint sig_id);
 
 #define default_handler_method_prefix "signal_do_"
@@ -47,10 +57,10 @@ static VALUE eNoSignalError;
 
 // FIXME: use rb_protect
 static gboolean
-accumulator_func(GSignalInvocationHint* ihint,
-                 GValue*                return_accu,
-                 const GValue*          handler_return,
-                 gpointer               data)
+accumulator_func(G_GNUC_UNUSED GSignalInvocationHint *ihint,
+                 GValue *return_accu,
+                 const GValue *handler_return,
+                 gpointer data)
 {
     VALUE proc = (VALUE)data;
     VALUE val = GVAL2RVAL(return_accu);
@@ -72,34 +82,97 @@ accumulator_func(GSignalInvocationHint* ihint,
     return continue_emission;
 }
 
+struct rval2gtypes_args {
+    VALUE ary;
+    long n;
+    GType *result;
+};
+
+static VALUE
+rbg_rval2gtypes_body(VALUE value)
+{
+    long i;
+    struct rval2gtypes_args *args = (struct rval2gtypes_args *)value;
+
+    for (i = 0; i < args->n; i++)
+        args->result[i] = rbgobj_gtype_get(RARRAY_PTR(args->ary)[i]);
+
+    return Qnil;
+}
+
+static G_GNUC_NORETURN VALUE
+rbg_rval2gtypes_rescue(VALUE value)
+{
+    g_free(((struct rval2gtypes_args *)value)->result);
+
+    rb_exc_raise(rb_errinfo());
+}
+
+static GType *
+rbg_rval2gtypes(volatile VALUE *value, long *n)
+{
+    struct rval2gtypes_args args;
+
+    args.ary = *value = rb_ary_dup(rb_ary_to_ary(*value));
+    args.n = RARRAY_LEN(args.ary);
+    args.result = g_new(GType, args.n + 1);
+
+    rb_rescue(rbg_rval2gtypes_body, (VALUE)&args,
+              rbg_rval2gtypes_rescue, (VALUE)&args);
+
+    if (n != NULL)
+        *n = args.n;
+
+    return args.result;
+}
+
+static GType *
+rbg_rval2gtypes_accept_nil(volatile VALUE *value, long *n)
+{
+    if (!NIL_P(*value))
+        return rbg_rval2gtypes(value, n);
+
+    if (n != NULL)
+        *n = 0;
+
+    return NULL;
+}
+
+#define RVAL2GTYPES(value, n) rbg_rval2gtypes(&(value), &(n))
+#define RVAL2GTYPES_ACCEPT_NIL(value, n) rbg_rval2gtypes_accept_nil(&(value), &(n))
+
 static VALUE
 gobj_s_signal_new(int argc, VALUE* argv, VALUE self)
 {
-    const RGObjClassInfo* cinfo = rbgobj_lookup_class(self);
-    VALUE signal_name, signal_flags, accumulator, return_type, params;
-    GClosure* class_closure;
-    GType* param_types;
-    long i, n_params;
-    guint sig;
+    const RGObjClassInfo *cinfo = rbgobj_lookup_class(self);
+    VALUE rbsignal_name, rbsignal_flags, accumulator, rbreturn_type, params;
+    const gchar *signal_name;
+    GSignalFlags signal_flags;
+    GClosure *class_closure;
+    GType return_type;
+    GType *param_types;
+    long n_params;
+    guint signal;
 
-    rb_scan_args(argc, argv, "4*", &signal_name, &signal_flags,
-                 &accumulator, &return_type, &params);
+    rb_scan_args(argc, argv, "4*",
+                 &rbsignal_name, &rbsignal_flags, &accumulator, &rbreturn_type, &params);
 
     if (cinfo->klass != self)
-        rb_raise(rb_eTypeError, "%s isn't registerd class",
+        rb_raise(rb_eTypeError, "not a registered class: %s",
                  rb_class2name(self));
 
-    if (SYMBOL_P(signal_name))
-        signal_name = rb_str_new2(rb_id2name(SYM2ID(signal_name)));
-    else
-        StringValue(signal_name);
+    if (SYMBOL_P(rbsignal_name))
+        rbsignal_name = rb_str_new2(rb_id2name(SYM2ID(rbsignal_name)));
+    signal_name = RVAL2CSTR(rbsignal_name);
+
+    signal_flags = NUM2INT(rbsignal_flags);
 
     {
         VALUE factory;
         VALUE proc;
         ID method_id;
 
-        method_id = rb_to_id(rb_str_concat(rb_str_new2(default_handler_method_prefix), signal_name));
+        method_id = rb_to_id(rb_str_concat(rb_str_new2(default_handler_method_prefix), rbsignal_name));
 
         factory = rb_eval_string(
           "lambda{|klass, id|\n"
@@ -110,37 +183,33 @@ gobj_s_signal_new(int argc, VALUE* argv, VALUE self)
         proc = rb_funcall(factory, rb_intern("call"), 2, self, ID2SYM(method_id));
 
         class_closure = g_rclosure_new(proc, Qnil, NULL);
+        /* TODO: Should this be done even if something below it fails? */
         g_rclosure_attach(class_closure, self);
     }
 
-    if (NIL_P(params)) {
-        n_params = 0;
-        param_types = NULL;
-    } else {
-        n_params = RARRAY_LEN(params);
-        param_types = ALLOCA_N(GType, n_params);
-        for (i = 0; i < n_params; i++)
-            param_types[i] = rbgobj_gtype_get(RARRAY_PTR(params)[i]);
-    }
+    return_type = rbgobj_gtype_get(rbreturn_type);
+    param_types = RVAL2GTYPES_ACCEPT_NIL(params, n_params);
 
-    sig = g_signal_newv(StringValuePtr(signal_name),
-                        cinfo->gtype,
-                        NUM2INT(signal_flags),
-                        class_closure,
-                        NIL_P(accumulator) ? NULL : accumulator_func,
-                        NIL_P(accumulator) ? NULL : (gpointer)accumulator,
-                        NULL, /* c_marshaller */
-                        rbgobj_gtype_get(return_type),
-                        n_params,
-                        param_types);
+    signal = g_signal_newv(signal_name,
+                           cinfo->gtype,
+                           signal_flags,
+                           class_closure,
+                           NIL_P(accumulator) ? NULL : accumulator_func,
+                           NIL_P(accumulator) ? NULL : (gpointer)accumulator,
+                           NULL, /* c_marshaller */
+                           return_type,
+                           n_params,
+                           param_types);
 
-    if (!sig)
+    g_free(param_types);
+
+    if (!signal)
         rb_raise(rb_eRuntimeError, "g_signal_newv failed");
 
     if (!NIL_P(accumulator))
         G_RELATIVE(self, accumulator); /* FIXME */
 
-    return rbgobj_signal_wrap(sig);
+    return rbgobj_signal_wrap(signal);
 }
 
 static void
@@ -190,7 +259,7 @@ gobj_s_signal(VALUE self, VALUE name)
         sig_name = rb_id2name(SYM2ID(name));
     else
         sig_name = StringValuePtr(name);
-    
+
     sig_id = g_signal_lookup(sig_name, CLASS2GTYPE(self));
     if (!sig_id)
         rb_raise(eNoSignalError, "no such signal: %s", sig_name);
@@ -225,7 +294,7 @@ static VALUE
 gobj_sig_connect_impl(gboolean after, int argc, VALUE *argv, VALUE self)
 {
     VALUE sig, rest;
-    int i;
+    gulong handler_id;
     GClosure* rclosure;
     const char* sig_name;
     guint signal_id;
@@ -256,9 +325,10 @@ gobj_sig_connect_impl(gboolean after, int argc, VALUE *argv, VALUE self)
                           sig_name);
     g_rclosure_set_tag((GClosure *)rclosure, tag);
     g_free(tag);
-    i = g_signal_connect_closure_by_id(g_object, signal_id, detail, rclosure, after);
+    handler_id = g_signal_connect_closure_by_id(g_object, signal_id, detail,
+                                                rclosure, after);
 
-    return INT2FIX(i);
+    return ULONG2NUM(handler_id);
 }
 
 static VALUE
@@ -298,7 +368,7 @@ struct emit_arg {
 static VALUE
 emit_body(struct emit_arg* arg)
 {
-    GValue param = { 0, };
+    GValue param = G_VALUE_INIT;
 
     g_value_init(&param, G_TYPE_FROM_INSTANCE(RVAL2GOBJ(arg->self)));
     rbgobj_rvalue_to_gvalue(arg->self, &param);
@@ -320,7 +390,7 @@ emit_body(struct emit_arg* arg)
 
     {
         gboolean use_ret = (arg->query.return_type != G_TYPE_NONE);
-        GValue return_value = {0,};
+        GValue return_value = G_VALUE_INIT;
 
         if (use_ret)
             g_value_init(&return_value,
@@ -376,7 +446,7 @@ gobj_sig_emit(int argc, VALUE *argv, VALUE self)
 
     arg.self = self;
     arg.instance_and_params = g_value_array_new(1 + arg.query.n_params);
-    
+
     return rb_ensure(emit_body, (VALUE)&arg, emit_ensure, (VALUE)&arg);
 }
 
@@ -444,34 +514,34 @@ gobj_sig_handler_is_connected(VALUE self, VALUE id)
 }
 
 #if 0
-gulong	 g_signal_handler_find		      (gpointer		  instance,
-					       GSignalMatchType	  mask,
-					       guint		  signal_id,
-					       GQuark		  detail,
-					       GClosure		 *closure,
-					       gpointer		  func,
-					       gpointer		  data);
-guint	 g_signal_handlers_block_matched      (gpointer		  instance,
-					       GSignalMatchType	  mask,
-					       guint		  signal_id,
-					       GQuark		  detail,
-					       GClosure		 *closure,
-					       gpointer		  func,
-					       gpointer		  data);
-guint	 g_signal_handlers_unblock_matched    (gpointer		  instance,
-					       GSignalMatchType	  mask,
-					       guint		  signal_id,
-					       GQuark		  detail,
-					       GClosure		 *closure,
-					       gpointer		  func,
-					       gpointer		  data);
-guint	 g_signal_handlers_disconnect_matched (gpointer		  instance,
-					       GSignalMatchType	  mask,
-					       guint		  signal_id,
-					       GQuark		  detail,
-					       GClosure		 *closure,
-					       gpointer		  func,
-					       gpointer		  data);
+gulong   g_signal_handler_find             (gpointer        instance,
+                           GSignalMatchType      mask,
+                           guint         signal_id,
+                           GQuark        detail,
+                           GClosure         *closure,
+                           gpointer          func,
+                           gpointer          data);
+guint    g_signal_handlers_block_matched      (gpointer         instance,
+                           GSignalMatchType      mask,
+                           guint         signal_id,
+                           GQuark        detail,
+                           GClosure         *closure,
+                           gpointer          func,
+                           gpointer          data);
+guint    g_signal_handlers_unblock_matched    (gpointer         instance,
+                           GSignalMatchType      mask,
+                           guint         signal_id,
+                           GQuark        detail,
+                           GClosure         *closure,
+                           gpointer          func,
+                           gpointer          data);
+guint    g_signal_handlers_disconnect_matched (gpointer         instance,
+                           GSignalMatchType      mask,
+                           guint         signal_id,
+                           GQuark        detail,
+                           GClosure         *closure,
+                           gpointer          func,
+                           gpointer          data);
 #endif
 
 static VALUE
@@ -493,7 +563,7 @@ chain_from_overridden_body(struct emit_arg* arg)
 
     {
         gboolean use_ret = (arg->query.return_type != G_TYPE_NONE);
-        GValue return_value = {0,};
+        GValue return_value = G_VALUE_INIT;
 
         if (use_ret)
             g_value_init(&return_value,
@@ -575,51 +645,12 @@ gobj_s_method_added(VALUE klass, VALUE id)
     {
         VALUE mod = rb_define_module_under(klass, RubyGObjectHookModule);
         rb_include_module(klass, mod);
-        rb_define_method(mod, name, gobj_sig_chain_from_overridden, -1);
+        rbg_define_method(mod, name, gobj_sig_chain_from_overridden, -1);
     }
 
     return Qnil;
 }
 
-static void
-Init_signal_misc()
-{
-    signal_func_table = rb_hash_new();
-    rb_global_variable(&signal_func_table);
-
-    rb_define_method(mMetaInterface, "signal_new", gobj_s_signal_new, -1);
-    rb_define_method(mMetaInterface, "signals", gobj_s_signals, -1);
-    rb_define_method(mMetaInterface, "signal", gobj_s_signal, 1);
-
-    rb_define_method(cInstantiatable, "signal_has_handler_pending?",
-                     gobj_sig_has_handler_pending, -1);
-    rb_define_method(cInstantiatable, "signal_connect", gobj_sig_connect, -1);
-    rb_define_method(cInstantiatable, "signal_connect_after",
-                     gobj_sig_connect_after, -1);
-
-#if 0
-    rb_define_method(cInstantiatable, "signal_invocation_hint",
-                     gobj_sig_get_invocation_hint, 0);
-#endif
-
-    rb_define_method(cInstantiatable, "signal_emit",
-                     gobj_sig_emit, -1);
-    rb_define_method(cInstantiatable, "signal_emit_stop",
-                     gobj_sig_emit_stop, 1);
-    rb_define_method(cInstantiatable, "signal_handler_block",
-                     gobj_sig_handler_block, 1);
-    rb_define_method(cInstantiatable, "signal_handler_unblock",
-                     gobj_sig_handler_unblock, 1);
-    rb_define_method(cInstantiatable, "signal_handler_disconnect",
-                     gobj_sig_handler_disconnect, 1);
-
-    rb_define_method(cInstantiatable, "signal_handler_is_connected?",
-                     gobj_sig_handler_is_connected, 1);
-
-    rb_define_singleton_method(cInstantiatable, "method_added",
-                               gobj_s_method_added, 1);
-}
-
 /**********************************************************************/
 
 VALUE
@@ -628,13 +659,13 @@ rbgobj_signal_wrap(guint sig_id)
     VALUE result;
     GSignalQuery* query;
 
-    result = Data_Make_Struct(cSignal, GSignalQuery, NULL, free, query);
+    result = Data_Make_Struct(RG_TARGET_NAMESPACE, GSignalQuery, NULL, free, query);
     g_signal_query(sig_id, query);
     return result;
 }
 
 static VALUE
-query_signal_id(VALUE self)
+rg_id(VALUE self)
 {
     GSignalQuery* query;
     Data_Get_Struct(self, GSignalQuery, query);
@@ -642,7 +673,7 @@ query_signal_id(VALUE self)
 }
 
 static VALUE
-query_signal_name(VALUE self)
+rg_name(VALUE self)
 {
     GSignalQuery* query;
     Data_Get_Struct(self, GSignalQuery, query);
@@ -650,7 +681,7 @@ query_signal_name(VALUE self)
 }
 
 static VALUE
-query_itype(VALUE self)
+rg_itype(VALUE self)
 {
     GSignalQuery* query;
     Data_Get_Struct(self, GSignalQuery, query);
@@ -658,7 +689,7 @@ query_itype(VALUE self)
 }
 
 static VALUE
-query_owner(VALUE self)
+rg_owner(VALUE self)
 {
     GSignalQuery* query;
     Data_Get_Struct(self, GSignalQuery, query);
@@ -666,7 +697,7 @@ query_owner(VALUE self)
 }
 
 static VALUE
-query_return_type(VALUE self)
+rg_return_type(VALUE self)
 {
     GSignalQuery* query;
     Data_Get_Struct(self, GSignalQuery, query);
@@ -674,7 +705,7 @@ query_return_type(VALUE self)
 }
 
 static VALUE
-query_signal_flags(VALUE self)
+rg_flags(VALUE self)
 {
     GSignalQuery* query;
     Data_Get_Struct(self, GSignalQuery, query);
@@ -682,7 +713,7 @@ query_signal_flags(VALUE self)
 }
 
 static VALUE
-query_param_types(VALUE self)
+rg_param_types(VALUE self)
 {
     GSignalQuery* query;
     VALUE result;
@@ -697,7 +728,7 @@ query_param_types(VALUE self)
 }
 
 static VALUE
-query_inspect(VALUE self)
+rg_inspect(VALUE self)
 {
     GSignalQuery* query;
     gchar* s;
@@ -734,7 +765,6 @@ query_is_flag(G_SIGNAL_DETAILED)
 query_is_flag(G_SIGNAL_ACTION)
 query_is_flag(G_SIGNAL_NO_HOOKS)
 
-
 static gboolean
 hook_func(GSignalInvocationHint* ihint,
           guint                  n_param_values,
@@ -742,7 +772,7 @@ hook_func(GSignalInvocationHint* ihint,
           gpointer               data)
 {
     GClosure* closure = data;
-    GValue ret_val ={0,};
+    GValue ret_val =G_VALUE_INIT;
     gboolean ret;
 
     g_value_init(&ret_val, G_TYPE_BOOLEAN);
@@ -768,7 +798,7 @@ g_signal_add_emission_hook_closure (guint     signal_id,
 }
 
 static VALUE
-signal_add_emission_hook(int argc, VALUE* argv, VALUE self)
+rg_add_emission_hook(int argc, VALUE* argv, VALUE self)
 {
     GSignalQuery* query;
     VALUE proc;
@@ -798,7 +828,7 @@ signal_add_emission_hook(int argc, VALUE* argv, VALUE self)
 }
 
 static VALUE
-signal_remove_emission_hook(VALUE self, VALUE hook_id)
+rg_remove_emission_hook(VALUE self, VALUE hook_id)
 {
     GSignalQuery* query;
     Data_Get_Struct(self, GSignalQuery, query);
@@ -806,56 +836,6 @@ signal_remove_emission_hook(VALUE self, VALUE hook_id)
     return Qnil;
 }
 
-static void
-Init_signal_class()
-{
-    VALUE cSignalFlags, cSignalMatchType;
-
-    cSignal = rb_define_class_under(mGLib, "Signal", rb_cData);
-
-    rb_define_method(cSignal, "id", query_signal_id, 0);
-    rb_define_method(cSignal, "name", query_signal_name, 0);
-    rb_define_method(cSignal, "flags", query_signal_flags, 0);
-    rb_define_method(cSignal, "itype", query_itype, 0);
-    rb_define_method(cSignal, "owner", query_owner, 0);
-    rb_define_method(cSignal, "return_type", query_return_type, 0);
-    rb_define_method(cSignal, "param_types", query_param_types, 0);
-    rb_define_method(cSignal, "inspect", query_inspect, 0);
-
-    rb_define_method(cSignal, "add_emission_hook", signal_add_emission_hook, -1);
-    rb_define_method(cSignal, "remove_emission_hook", signal_remove_emission_hook, 1);
-
-    /* GSignalFlags */
-    cSignalFlags = G_DEF_CLASS(G_TYPE_SIGNAL_FLAGS, "SignalFlags", mGLib);
-    G_DEF_CONSTANTS(cSignal, G_TYPE_SIGNAL_FLAGS, "G_SIGNAL_");
-    rb_define_const(cSignalFlags, "MASK", INT2NUM(G_SIGNAL_FLAGS_MASK));
-    rb_define_const(cSignal, "FLAGS_MASK", INT2NUM(G_SIGNAL_FLAGS_MASK));
-
-    rb_define_method(cSignal, "run_first?", query_is_G_SIGNAL_RUN_FIRST, 0);
-    rb_define_method(cSignal, "run_last?", query_is_G_SIGNAL_RUN_LAST, 0);
-    rb_define_method(cSignal, "run_cleanup?", query_is_G_SIGNAL_RUN_CLEANUP, 0);
-    rb_define_method(cSignal, "no_recurse?", query_is_G_SIGNAL_NO_RECURSE, 0);
-    rb_define_method(cSignal, "detailed?", query_is_G_SIGNAL_DETAILED, 0);
-    rb_define_method(cSignal, "action?", query_is_G_SIGNAL_ACTION, 0);
-    rb_define_method(cSignal, "no_hooks?", query_is_G_SIGNAL_NO_HOOKS, 0);
-
-
-    /* GConnectFlags */
-    G_DEF_CLASS(G_TYPE_CONNECT_FLAGS, "ConnectFlags", mGLib);
-    G_DEF_CONSTANTS(cSignal, G_TYPE_CONNECT_FLAGS, "G_");
-
-    /* GSignalMatchType */
-    cSignalMatchType = G_DEF_CLASS(G_TYPE_SIGNAL_MATCH_TYPE,
-				   "SignalMatchType", mGLib);
-    G_DEF_CONSTANTS(cSignal, G_TYPE_SIGNAL_MATCH_TYPE, "G_SIGNAL_");
-    rb_define_const(cSignalMatchType, "MASK", INT2NUM(G_SIGNAL_MATCH_MASK));
-    rb_define_const(cSignal, "MATCH_MASK", INT2NUM(G_SIGNAL_MATCH_MASK));
-
-    rb_define_const(cSignal, "TYPE_STATIC_SCOPE", INT2FIX(G_SIGNAL_TYPE_STATIC_SCOPE));
-
-    eNoSignalError = rb_define_class_under(mGLib, "NoSignalError", rb_eNameError);
-}
-
 /**********************************************************************/
 
 void
@@ -897,7 +877,7 @@ rbgobj_define_action_methods(VALUE klass)
                 source,
                 "def %s(%s)\n  signal_emit('%s'%s)\nend\n",
                 method_name,
-                (query.n_params > 0) ? args->str + 1 : "", // hack
+                (query.n_params > 0) ? args->str + 1 : "", // Skip initial ','
                 query.signal_name,
                 args->str);
 
@@ -914,9 +894,85 @@ rbgobj_define_action_methods(VALUE klass)
 /**********************************************************************/
 
 void
-Init_gobject_gsignal()
+Init_gobject_gsignal(void)
 {
-    Init_signal_class();
-    Init_signal_misc();
-}
+    VALUE cSignalFlags, cSignalMatchType;
 
+    RG_TARGET_NAMESPACE = rb_define_class_under(mGLib, "Signal", rb_cData);
+
+    RG_DEF_METHOD(id, 0);
+    RG_DEF_METHOD(name, 0);
+    RG_DEF_METHOD(flags, 0);
+    RG_DEF_METHOD(itype, 0);
+    RG_DEF_METHOD(owner, 0);
+    RG_DEF_METHOD(return_type, 0);
+    RG_DEF_METHOD(param_types, 0);
+    RG_DEF_METHOD(inspect, 0);
+
+    RG_DEF_METHOD(add_emission_hook, -1);
+    RG_DEF_METHOD(remove_emission_hook, 1);
+
+    /* GSignalFlags */
+    cSignalFlags = G_DEF_CLASS(G_TYPE_SIGNAL_FLAGS, "SignalFlags", mGLib);
+    G_DEF_CONSTANTS(RG_TARGET_NAMESPACE, G_TYPE_SIGNAL_FLAGS, "G_SIGNAL_");
+    rb_define_const(cSignalFlags, "MASK", INT2NUM(G_SIGNAL_FLAGS_MASK));
+    rb_define_const(RG_TARGET_NAMESPACE, "FLAGS_MASK", INT2NUM(G_SIGNAL_FLAGS_MASK));
+
+    rbg_define_method(RG_TARGET_NAMESPACE, "run_first?", query_is_G_SIGNAL_RUN_FIRST, 0);
+    rbg_define_method(RG_TARGET_NAMESPACE, "run_last?", query_is_G_SIGNAL_RUN_LAST, 0);
+    rbg_define_method(RG_TARGET_NAMESPACE, "run_cleanup?", query_is_G_SIGNAL_RUN_CLEANUP, 0);
+    rbg_define_method(RG_TARGET_NAMESPACE, "no_recurse?", query_is_G_SIGNAL_NO_RECURSE, 0);
+    rbg_define_method(RG_TARGET_NAMESPACE, "detailed?", query_is_G_SIGNAL_DETAILED, 0);
+    rbg_define_method(RG_TARGET_NAMESPACE, "action?", query_is_G_SIGNAL_ACTION, 0);
+    rbg_define_method(RG_TARGET_NAMESPACE, "no_hooks?", query_is_G_SIGNAL_NO_HOOKS, 0);
+
+    /* GConnectFlags */
+    G_DEF_CLASS(G_TYPE_CONNECT_FLAGS, "ConnectFlags", mGLib);
+    G_DEF_CONSTANTS(RG_TARGET_NAMESPACE, G_TYPE_CONNECT_FLAGS, "G_");
+
+    /* GSignalMatchType */
+    cSignalMatchType = G_DEF_CLASS(G_TYPE_SIGNAL_MATCH_TYPE,
+                                   "SignalMatchType", mGLib);
+    G_DEF_CONSTANTS(RG_TARGET_NAMESPACE, G_TYPE_SIGNAL_MATCH_TYPE, "G_SIGNAL_");
+    rb_define_const(cSignalMatchType, "MASK", INT2NUM(G_SIGNAL_MATCH_MASK));
+    rb_define_const(RG_TARGET_NAMESPACE, "MATCH_MASK", INT2NUM(G_SIGNAL_MATCH_MASK));
+
+    rb_define_const(RG_TARGET_NAMESPACE, "TYPE_STATIC_SCOPE", INT2FIX(G_SIGNAL_TYPE_STATIC_SCOPE));
+
+    eNoSignalError = rb_define_class_under(mGLib, "NoSignalError", rb_eNameError);
+
+    signal_func_table = rb_hash_new();
+    rb_global_variable(&signal_func_table);
+
+    rbg_define_method(mMetaInterface, "signal_new", gobj_s_signal_new, -1);
+    rbg_define_method(mMetaInterface, "signals", gobj_s_signals, -1);
+    rbg_define_method(mMetaInterface, "signal", gobj_s_signal, 1);
+
+    rbg_define_method(cInstantiatable, "signal_has_handler_pending?",
+                     gobj_sig_has_handler_pending, -1);
+    rbg_define_method(cInstantiatable, "signal_connect", gobj_sig_connect, -1);
+    rbg_define_method(cInstantiatable, "signal_connect_after",
+                     gobj_sig_connect_after, -1);
+
+#if 0
+    rbg_define_method(cInstantiatable, "signal_invocation_hint",
+                     gobj_sig_get_invocation_hint, 0);
+#endif
+
+    rbg_define_method(cInstantiatable, "signal_emit",
+                     gobj_sig_emit, -1);
+    rbg_define_method(cInstantiatable, "signal_emit_stop",
+                     gobj_sig_emit_stop, 1);
+    rbg_define_method(cInstantiatable, "signal_handler_block",
+                     gobj_sig_handler_block, 1);
+    rbg_define_method(cInstantiatable, "signal_handler_unblock",
+                     gobj_sig_handler_unblock, 1);
+    rbg_define_method(cInstantiatable, "signal_handler_disconnect",
+                     gobj_sig_handler_disconnect, 1);
+
+    rbg_define_method(cInstantiatable, "signal_handler_is_connected?",
+                     gobj_sig_handler_is_connected, 1);
+
+    rbg_define_singleton_method(cInstantiatable, "method_added",
+                               gobj_s_method_added, 1);
+}
