@@ -77,9 +77,10 @@ gpointer
 rbgobj_instance_from_ruby_object(VALUE obj)
 {
     GType type;
+    GType fundamental_type;
 
     if (NIL_P(obj))
-    	return NULL;
+        return NULL;
 
     type = RVAL2GTYPE(obj);
     if (rbgobj_convert_has_type(type)) {
@@ -88,16 +89,18 @@ rbgobj_instance_from_ruby_object(VALUE obj)
             return instance;
     }
 
-    type = G_TYPE_FUNDAMENTAL(type);
-    switch (type){
+    fundamental_type = G_TYPE_FUNDAMENTAL(type);
+    switch (fundamental_type) {
     case G_TYPE_OBJECT:
         return rbgobj_get_gobject(obj);
+    case G_TYPE_BOXED:
+        return rbgobj_boxed_get(obj, type);
     case G_TYPE_PARAM:
         return rbgobj_get_param_spec(obj);
     default:
       {
         gpointer instance;
-        if (!rbgobj_convert_robj2instance(type, obj, &instance)) {
+        if (!rbgobj_convert_robj2instance(fundamental_type, obj, &instance)) {
             rb_raise(rb_eTypeError, "%s isn't supported",
                      rb_class2name(CLASS_OF(obj)));
         }
@@ -146,25 +149,31 @@ rbgobj_ruby_object_from_instance2(gpointer instance, gboolean alloc)
     }
 }
 
+void
+rbgobj_instance_unref(gpointer instance)
+{
+    GType type;
+
+    type = G_TYPE_FROM_INSTANCE(instance);
+    if (!rbgobj_convert_unref(type, instance)) {
+        type = G_TYPE_FUNDAMENTAL(type);
+        switch (type) {
+          case G_TYPE_OBJECT:
+            g_object_unref(instance);
+            break;
+          default:
+            rbgobj_convert_unref(type, instance);
+            break;
+        }
+    }
+}
+
 VALUE
 rbgobj_ruby_object_from_instance_with_unref(gpointer instance)
 {
     VALUE result = rbgobj_ruby_object_from_instance(instance);
     if (!NIL_P(result)) {
-        GType type;
-
-        type = G_TYPE_FROM_INSTANCE(instance);
-        if (!rbgobj_convert_unref(type, instance)) {
-            type = G_TYPE_FUNDAMENTAL(type);
-            switch (type) {
-              case G_TYPE_OBJECT:
-                g_object_unref(instance);
-                break;
-              default:
-                rbgobj_convert_unref(type, instance);
-                break;
-            }
-        }
+        rbgobj_instance_unref(instance);
     }
     return result;
 }
@@ -179,7 +188,7 @@ rbgobj_add_relative(VALUE obj, VALUE relative)
     if (RVAL2CBOOL(rb_ivar_defined(obj, id_relatives)))
         hash = rb_ivar_get(obj, id_relatives);
 
-    if (NIL_P(hash) || TYPE(hash) != T_HASH) {
+    if (NIL_P(hash) || TYPE(hash) != RUBY_T_HASH) {
         hash = rb_hash_new();
         rb_ivar_set(obj, id_relatives, hash);
     }
@@ -203,7 +212,7 @@ rbgobj_add_relative_removable(VALUE obj, VALUE relative, ID obj_ivar_id, VALUE h
     if (RVAL2CBOOL(rb_ivar_defined(obj, obj_ivar_id)))
         hash = rb_ivar_get(obj, obj_ivar_id);
 
-    if (NIL_P(hash) || TYPE(hash) != T_HASH) {
+    if (NIL_P(hash) || TYPE(hash) != RUBY_T_HASH) {
         hash = rb_hash_new();
         rb_ivar_set(obj, obj_ivar_id, hash);
     }
@@ -218,7 +227,7 @@ rbgobj_get_relative_removable(VALUE obj, ID obj_ivar_id, VALUE hash_key)
     if (RVAL2CBOOL(rb_ivar_defined(obj, obj_ivar_id)))
         hash = rb_ivar_get(obj, obj_ivar_id);
 
-    if (NIL_P(hash) || TYPE(hash) != T_HASH) {
+    if (NIL_P(hash) || TYPE(hash) != RUBY_T_HASH) {
         return Qnil;
     }
     return rb_hash_aref(hash, hash_key);
@@ -232,7 +241,7 @@ rbgobj_remove_relative(VALUE obj, ID obj_ivar_id, VALUE hash_key)
     if (RVAL2CBOOL(rb_ivar_defined(obj, obj_ivar_id)))
         hash = rb_ivar_get(obj, obj_ivar_id);
 
-    if (NIL_P(hash) || TYPE(hash) != T_HASH) {
+    if (NIL_P(hash) || TYPE(hash) != RUBY_T_HASH) {
         /* should not happen. */
     } else {
         rb_funcall(hash, id_delete, 1, hash_key);
@@ -262,11 +271,9 @@ rbgobj_define_property_accessors(VALUE klass)
     gtype = CLASS2GTYPE(klass);
 
     if (G_TYPE_IS_INTERFACE(gtype)){
-#if GLIB_CHECK_VERSION(2,4,0)
         gpointer iface = g_type_default_interface_ref(gtype);
         pspecs = g_object_interface_list_properties(iface, &n_properties);
         g_type_default_interface_unref(iface);
-#endif
     } else {
         GObjectClass* oclass = G_OBJECT_CLASS(g_type_class_ref(gtype));
         pspecs = g_object_class_list_properties(oclass, &n_properties);
@@ -366,9 +373,7 @@ Init_gobject(void)
     Init_gobject_gvalue();
     Init_gobject_gvaluetypes();
     Init_gobject_gboxed();
-#if GLIB_CHECK_VERSION(2,6,0)
     Init_gobject_gstrv();
-#endif
     Init_gobject_value_array();
     Init_gobject_genumflags();
     Init_gobject_gparam();
