@@ -16,9 +16,51 @@ require 'glib-mkenums'
 
 $CFLAGS += " #{ENV['CFLAGS']}" if ENV['CFLAGS']
 
+def gcc?
+  CONFIG["GCC"] == "yes"
+end
+
+def disable_optimization_build_flag(flags)
+  if gcc?
+    flags.gsub(/(^|\s)?-O\d(\s|$)?/, '\\1-O0\\2')
+  else
+    flags
+  end
+end
+
+def enable_debug_build_flag(flags)
+  if gcc?
+    debug_option_pattern = /(^|\s)?-g\d?(\s|$)?/
+    if debug_option_pattern =~ flags
+      flags.gsub(debug_option_pattern, '\\1-g3\\2')
+    else
+      flags + " -g3"
+    end
+  else
+    flags
+  end
+end
+
+checking_for(checking_message("--enable-debug-build option")) do
+  enable_debug_build = enable_config("debug-build", false)
+  if enable_debug_build
+    $CFLAGS = disable_optimization_build_flag($CFLAGS)
+    $CFLAGS = enable_debug_build_flag($CFLAGS)
+
+    CONFIG["CXXFLAGS"] = disable_optimization_build_flag(CONFIG["CXXFLAGS"])
+    CONFIG["CXXFLAGS"] = enable_debug_build_flag(CONFIG["CXXFLAGS"])
+  end
+  enable_debug_build
+end
+
 def try_compiler_option(opt, &block)
   checking_for "#{opt} option to compiler" do
-    $CFLAGS += " #{opt}" if try_compile '', opt, &block
+    if try_compile '', opt + " -Werror", &block
+      $CFLAGS += " #{opt}"
+      true
+    else
+      false
+    end
   end
 end
 
@@ -52,6 +94,7 @@ try_compiler_option '-Wswitch-enum'
 try_compiler_option '-Wundef'
 # NOTE: Incredible amounts of false positives.
 #try_compiler_option '-Wunreachable-code'
+try_compiler_option '-Wout-of-line-declaration'
 try_compiler_option '-Wunsafe-loop-optimizations'
 try_compiler_option '-Wwrite-strings'
 
@@ -66,8 +109,8 @@ if ENV['GTK_BASEPATH'] and /cygwin/ !~ RUBY_PLATFORM
   $CFLAGS += " -I#{include_path} "
 end
 
-def setup_win32(target_name, base_dir=nil)
-  checking_for(checking_message("Win32 OS")) do
+def setup_windows(target_name, base_dir=nil)
+  checking_for(checking_message("Windows")) do
     case RUBY_PLATFORM
     when /cygwin|mingw|mswin/
       import_library_name = "libruby-#{target_name}.a"
@@ -87,6 +130,10 @@ def setup_win32(target_name, base_dir=nil)
     end
   end
 end
+# For backward compatibility
+def setup_win32(*args, &block)
+  setup_windows(*args, &block)
+end
 
 def find_gem_spec(package)
   begin
@@ -142,6 +189,17 @@ def add_depend_package_path(target_name, target_source_dir, target_build_dir)
     $INCFLAGS = "-I#{target_source_dir} #{$INCFLAGS}"
   end
 
+  case RUBY_PLATFORM
+  when /cygwin|mingw|mswin/
+    target_base_dir = Pathname.new(target_source_dir).parent.parent
+    target_binary_base_dir = target_base_dir + "vendor" + "local"
+    if target_binary_base_dir.exist?
+      $INCFLAGS = "-I#{target_binary_base_dir}/include #{$INCFLAGS}"
+      target_pkg_config_dir = target_binary_base_dir + "lib" + "pkgconfig"
+      PKGConfig.add_path(target_pkg_config_dir.to_s)
+    end
+  end
+
   return unless File.exist?(target_build_dir)
   if target_source_dir != target_build_dir
     $INCFLAGS = "-I#{target_build_dir} #{$INCFLAGS}"
@@ -158,13 +216,6 @@ def add_depend_package_path(target_name, target_source_dir, target_build_dir)
       $DLDFLAGS << " /libpath:#{target_build_dir}"
       $libs << " lib#{library_base_name}.lib"
     end
-    target_base_dir = Pathname.new(target_source_dir).parent.parent
-    target_binary_base_dir = target_base_dir + "vendor" + "local"
-    if target_binary_base_dir.exist?
-      $INCFLAGS = "-I#{target_binary_base_dir}/include #{$INCFLAGS}"
-      target_pkg_config_dir = target_binary_base_dir + "lib" + "pkgconfig"
-      PKGConfig.add_path(target_pkg_config_dir.to_s)
-    end
   end
 end
 
@@ -399,7 +450,27 @@ def glib_mkenums(prefix, files, g_type_prefix, include_files, options={})
 end
 
 def check_cairo(options={})
+  is_windows = (/mingw|cygwin|mswin/ === RUBY_PLATFORM)
+
   rcairo_source_dir = options[:rcairo_source_dir]
+  if rcairo_source_dir.nil?
+    suffix = nil
+    if is_windows
+      case RUBY_PLATFORM
+      when /\Ax86-mingw/
+        suffix = "win32"
+      when /\Ax64-mingw/
+        suffix = "win64"
+      end
+    end
+    rcairo_source_base_dir = "rcairo"
+    rcairo_source_base_dir << ".#{suffix}" if suffix
+    top_dir = options[:top_dir]
+    if top_dir
+      rcairo_source_dir = File.join(top_dir, "..", rcairo_source_base_dir)
+    end
+  end
+
   if rcairo_source_dir and !File.exist?(rcairo_source_dir)
     rcairo_source_dir = nil
   end
@@ -409,7 +480,7 @@ def check_cairo(options={})
   end
 
   unless rcairo_source_dir.nil?
-    if /mingw|cygwin|mswin/ =~ RUBY_PLATFORM
+    if is_windows
       options = {}
       build_dir = "tmp/#{RUBY_PLATFORM}/cairo/#{RUBY_VERSION}"
       if File.exist?(File.join(rcairo_source_dir, build_dir))
@@ -431,6 +502,8 @@ def package_platform
     :fedora
   elsif File.exist?("/etc/redhat-release")
     :redhat
+  elsif File.exist?("/etc/SuSE-release")
+    :suse
   elsif find_executable("brew")
     :homebrew
   elsif find_executable("port")
@@ -448,6 +521,7 @@ def normalize_native_package_info(native_package_info)
   native_package_info ||= {}
   native_package_info = native_package_info.dup
   native_package_info[:fedora] ||= native_package_info[:redhat]
+  native_package_info[:suse] ||= native_package_info[:fedora]
   native_package_info
 end
 
@@ -457,17 +531,21 @@ def install_missing_native_package(native_package_info)
   package = native_package_info[platform]
   return false if package.nil?
 
+  package_name, *options = package
+  package_command_line = [package_name, *options].join(" ")
   need_super_user_priviledge = true
   case platform
   when :debian
-    install_command = "apt-get install -V -y #{package}"
+    install_command = "apt-get install -V -y #{package_command_line}"
   when :fedora, :redhat
-    install_command = "yum install -y #{package}"
+    install_command = "yum install -y #{package_command_line}"
+  when :suse
+    install_command = "zypper --non-interactive install #{package_command_line}"
   when :homebrew
     need_super_user_priviledge = false
-    install_command = "brew install #{package}"
+    install_command = "brew install #{package_command_line}"
   when :macports
-    install_command = "port install -y #{package}"
+    install_command = "port install -y #{package_command_line}"
   else
     return false
   end
@@ -477,14 +555,14 @@ def install_missing_native_package(native_package_info)
     sudo = find_executable("sudo")
   end
 
-  installing_message = "installing '#{package}' native package... "
+  installing_message = "installing '#{package_name}' native package... "
   message("%s", installing_message)
   failed_to_get_super_user_priviledge = false
   if have_priviledge
     succeeded = xsystem(install_command)
   else
     if sudo
-      prompt = "[sudo] password for %u to install <#{package}>: "
+      prompt = "[sudo] password for %u to install <#{package_name}>: "
       sudo_options = "-p #{Shellwords.escape(prompt)}"
       install_command = "#{sudo} #{sudo_options} #{install_command}"
       succeeded = xsystem(install_command)
@@ -508,7 +586,7 @@ def install_missing_native_package(native_package_info)
   unless succeeded
     if failed_to_get_super_user_priviledge
       error_message = <<-EOM
-'#{package}' native package is required.
+'#{package_name}' native package is required.
 run the following command as super user to install required native package:
   \# #{install_command}
 EOM
@@ -557,4 +635,3 @@ check_ruby_func
 if /mingw/ =~ RUBY_PLATFORM
   $ruby.gsub!('\\', '/')
 end
-
