@@ -120,7 +120,8 @@ module GObjectIntrospection
                                          :parent => options[:parent])
       else
         klass = self.class.define_class(info.gtype, info.name, @base_module,
-                                        :parent => options[:parent])
+                                        :parent => options[:parent],
+                                        :size   => info.size)
       end
       load_fields(info, klass)
       load_methods(info, klass)
@@ -134,6 +135,7 @@ module GObjectIntrospection
 
     def define_boxed(info)
       klass = self.class.define_class(info.gtype, info.name, @base_module)
+      _ = klass # TODO: Remove me. It is just for suppressing a warning.
       # TODO
       # load_fields(info, klass)
       # load_methods(info, klass)
@@ -223,7 +225,12 @@ module GObjectIntrospection
       flags = field_info.flags
 
       if flags.readable?
-        klass.__send__(:define_method, name) do ||
+        if field_info.type.tag == TypeTag::BOOLEAN
+          reader_method_name = "#{name}?"
+        else
+          reader_method_name = name
+        end
+        klass.__send__(:define_method, reader_method_name) do ||
           info.get_field_value(self, i)
         end
       end
@@ -291,9 +298,9 @@ module GObjectIntrospection
 
       detail = "#{arguments.size} for "
       if n_in_args == n_required_in_args
-        detail << "#{info.n_in_args}"
+        detail << "#{n_in_args}"
       else
-        detail << "#{info.n_required_in_args}..#{info.n_in_args}"
+        detail << "#{n_required_in_args}..#{n_in_args}"
       end
       message = "#{method_name}: wrong number of arguments (#{detail})"
       raise ArgumentError, message
@@ -304,11 +311,12 @@ module GObjectIntrospection
       max_n_args = nil
       candidate_infos = []
       infos.each do |info|
-        if arguments.size == info.n_in_args
+        n_in_args = info.n_in_args
+        n_required_in_args = info.n_required_in_args
+        if (n_required_in_args..n_in_args).cover?(arguments.size)
           candidate_infos << info
         end
-        n_in_args = info.n_in_args
-        min_n_args = [min_n_args || n_in_args, n_in_args].min
+        min_n_args = [min_n_args || n_required_in_args, n_required_in_args].min
         max_n_args = [max_n_args || n_in_args, n_in_args].max
       end
 
@@ -316,8 +324,9 @@ module GObjectIntrospection
         return candidate_infos.first
       elsif candidate_infos.size > 1
         candidate_info = candidate_infos.find do |info|
-          info.in_args.each.with_index.all? do |arg_info, i|
-            match_argument?(arg_info, arguments[i])
+          in_arg_infos = info.in_args
+          arguments.each.with_index.all? do |argument, i|
+            match_argument?(in_arg_infos[i], argument)
           end
         end
         return candidate_info || candidate_infos.first
@@ -340,13 +349,22 @@ module GObjectIntrospection
       end
     end
 
-    def rubyish_method_name(function_info)
+    def rubyish_method_name(function_info, options={})
       name = function_info.name
+      if options[:prefix]
+        name = name.gsub(/\A#{Regexp.escape(options[:prefix])}/, "")
+      end
       return_type = function_info.return_type
       if return_type.tag == GObjectIntrospection::TypeTag::BOOLEAN
         case name
-        when /\A(?:is|get_is|get)_/
+        when /\A(?:is|get_is)_/
           "#{$POSTMATCH}?"
+        when /\Aget_/
+          if function_info.n_in_args.zero?
+            "#{$POSTMATCH}?"
+          else
+            name
+          end
         when /\A(?:has|use)_/
           "#{name}?"
         else
@@ -374,32 +392,51 @@ module GObjectIntrospection
       infos.each do |info|
         method_name = rubyish_method_name(info)
         load_method_info(info, klass, method_name)
-        if /\Aset_/ =~ method_name and info.n_args == 1
-          klass.__send__(:alias_method, "#{$POSTMATCH}=", method_name)
-        end
       end
     end
 
     def load_method_info(info, klass, method_name)
+      define_method(info, klass, method_name)
+      define_equal_style_setter(info, klass, method_name)
+    end
+
+    def define_method(info, klass, method_name)
       unlock_gvl = should_unlock_gvl?(info, klass)
       validate = lambda do |arguments|
         validate_arguments(info, "#{klass}\##{method_name}", arguments)
       end
+      if klass.method_defined?(method_name) and
+          klass.instance_method(method_name).owner == klass
+        klass.__send__(:remove_method, method_name)
+      end
+      function_info_p = (info.class == FunctionInfo)
       klass.__send__(:define_method, method_name) do |*arguments, &block|
+        arguments = [self] + arguments if function_info_p
         validate.call(arguments, &block)
         if block.nil? and info.require_callback?
-          Enumerator.new(self, method_name, *arguments)
+          to_enum(method_name, *arguments)
         else
-          info.invoke({
-                        :receiver  => self,
-                        :arguments => arguments,
-                        :unlock_gvl => unlock_gvl,
-                      },
-                      &block)
+          options = {
+            :arguments => arguments,
+            :unlock_gvl => unlock_gvl,
+          }
+          options[:receiver] = self unless function_info_p
+          info.invoke(options, &block)
         end
       end
     end
 
+    def define_equal_style_setter(info, klass, method_name)
+      if /\Aset_/ =~ method_name and info.n_args == 1
+        setter_method_name = "#{$POSTMATCH}="
+        if klass.method_defined?(setter_method_name) and
+            klass.instance_method(setter_method_name).owner == klass
+          klass.__send__(:remove_method, setter_method_name)
+        end
+        klass.__send__(:alias_method, setter_method_name, method_name)
+      end
+    end
+
     def load_function_infos(infos, klass)
       infos.each do |info|
         name = rubyish_method_name(info)
