@@ -1,6 +1,6 @@
 /* -*- c-file-style: "ruby"; indent-tabs-mode: nil -*- */
 /*
- *  Copyright (C) 2011  Ruby-GNOME2 Project Team
+ *  Copyright (C) 2011-2013  Ruby-GNOME2 Project Team
  *  Copyright (C) 2002-2005 Masao Mutoh
  *
  *  This library is free software; you can redistribute it and/or
@@ -39,7 +39,6 @@ rg_initialize(int argc, VALUE *argv, VALUE self)
     }
 
     G_INITIALIZE(self, description);
-    pango_font_description_free(description);
 
     return Qnil;
 }
@@ -151,7 +150,6 @@ rg_size(VALUE self)
     return INT2NUM(pango_font_description_get_size(_SELF(self)));
 }
 
-#if PANGO_CHECK_VERSION(1,8,0)
 static VALUE
 rg_set_absolute_size(VALUE self, VALUE size)
 {
@@ -164,7 +162,6 @@ rg_size_is_absolute_p(VALUE self)
 {
     return CBOOL2RVAL(pango_font_description_get_size_is_absolute(_SELF(self)));
 }
-#endif
 
 #if PANGO_CHECK_VERSION(1,16,0)
 static VALUE
@@ -249,11 +246,8 @@ Init_pango_font_description(VALUE mPango)
     RG_DEF_METHOD(stretch, 0);
     RG_DEF_METHOD(set_size, 1);
     RG_DEF_METHOD(size, 0);
-
-#if PANGO_CHECK_VERSION(1,8,0)
     RG_DEF_METHOD(set_absolute_size, 1);
     RG_DEF_METHOD_P(size_is_absolute, 0);
-#endif
 #if PANGO_CHECK_VERSION(1,16,0)
     RG_DEF_METHOD(set_gravity, 1);
     RG_DEF_METHOD(gravity, 0);
