@@ -1,18 +1,18 @@
+# -*- coding: utf-8 -*-
+
 =begin
   item.rb - Ruby/Pango sample script.
 
   Copyright (c) 2005 Ruby-GNOME2 Project
   This program is licenced under the same licence as Ruby-GNOME2.
 
-  $Id: item.rb,v 1.1 2005/09/17 17:09:12 mutoh Exp $
+  $Id: item.rb,v 1.3 2006/06/17 13:37:01 mutoh Exp $
 =end
 
 require 'gtk2'
 
-Gtk.init
-
 str = "Ruby-GNOME2こんにちわ" 
-win = Gtk::Window.new
+win = Gtk::Window.new("Ruby/Pango sample")
 win.realize
 
 p screen = Gdk::Screen.default
@@ -31,7 +31,7 @@ items.each do |v|
   ana = v.analysis
   puts "shape_engine = #{ana.shape_engine}, lang_engine = #{ana.lang_engine}, font = #{ana.font}, level = #{ana.level}"
   puts "language = #{ana.language}"
-  p ana.font.get_glyph_extents("a"[0])[0].to_a
+  p ana.font.get_glyph_extents("a".codepoints.first)[0].to_a
   ana.set_extra_attrs(attrs)
   p ana.extra_attrs
 end
