@@ -30,7 +30,7 @@ rescue LoadError
   require 'mkmf-gnome2'
 end
 
-["glib2", "atk", "pango", "gdk_pixbuf2"].each do |package|
+["glib2", "atk", "gdk_pixbuf2"].each do |package|
   directory = "#{package}#{version_suffix}"
   build_dir = "#{directory}/tmp/#{RUBY_PLATFORM}/#{package}/#{RUBY_VERSION}"
   add_depend_package(package, "#{directory}/ext/#{package}",
@@ -39,29 +39,16 @@ end
                      :target_build_dir => build_dir)
 end
 
-rcairo_options = {}
-rcairo_source_dir_names = ["rcairo"]
-if /mingw|cygwin|mswin/ =~ RUBY_PLATFORM
-  rcairo_source_dir_names.unshift("rcairo.win32")
-end
-rcairo_source_dir_names.each do |rcairo_source_dir_name|
-  rcairo_source_dir = top_dir.parent.expand_path + rcairo_source_dir_name
-  if rcairo_source_dir.exist?
-    rcairo_options[:rcairo_source_dir] = rcairo_source_dir.to_s
-    break
-  end
-end
-
-unless check_cairo(rcairo_options)
+unless check_cairo(:top_dir => top_dir)
   exit(false)
 end
 
-setup_win32(module_name, base_dir)
+setup_windows(module_name, base_dir)
 
 unless required_pkg_config_package([package_id, 0, 12, 0],
                                    :debian => "libpoppler-glib-dev",
                                    :redhat => "poppler-glib-devel",
-#                                   :homebrew => "poppler", # need --with-glib option
+                                   :homebrew => "poppler",
                                    :macports => "poppler")
   exit(false)
 end
