@@ -28,10 +28,9 @@ Pathname.glob((base_dir + "*").to_s) do |dir|
 end
 
 ignored_modules = [
-  "gstreamer",
   "gstreamer-no-gi",
-  "webkit-gtk",
-  "webkit-gtk2",
+  "gdk3-no-gi",
+  "gtk3-gi",
 ]
 
 succeeded = true
